/**
  * Copyright (C) 2016 Ortus Solutions, Corp
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALING
 * IN THE SOFTWARE.
 */
component extends='lucee.admin.plugin.Plugin' {
	
	/**
	* This function will be called once to initalize the plugin
	*/
	function init( struct lang, struct app ) {
		variables.licenseHelper = createObject( 'java', 'ortus.extension.cache.util.LicenseHelper' );
		variables.trialExpiration = variables.licenseHelper.getTrialExpiration();
		
		variables.activationServer = "https://www.ortussolutions.com/store/activate/index";
		try{
			variables.hostName = createObject( "java", "java.net.InetAddress" ).getLocalHost().getHostName();
		} catch( Any e ) {
			variables.hostName = cgi.http_host;
		}
		variables.productSKU 		= "mongodb-lucee-5-ext";
		variables.productVersion 	= "1.0.0.2213";
		
	}

	/**
	* This appears to be the default action name by convention
	* 
	* @lang A struct of translated keys based on selected language
	* @app A struct of data for this plugin persisted in the application scope
	* @req A struct containing  form and url variables for the request.
	*/
	function overview( struct lang, struct app, struct req ) {
		req.licenseProperties 	= getLicenseProperties();
		req.trialExpiration 	= variables.trialExpiration;
		// Struct with keys "valid" and "message".
		req.licenseValidCheck 	= variables.licenseHelper.validateLicense( req.licenseProperties );
		req.activationServer 	= variables.activationServer;
	}

	/**
	* Update Note
	* 
	* @lang A struct of translated keys based on selected language
	* @app A struct of data for this plugin persisted in the application scope() and save()
	* @req A struct containing  form and url variables for the request.
	*/
	function update( struct lang, struct app, struct req ) {
		var props = getLicenseProperties();
		var licenseProperties = getLicenseProperties();
		var licenseValidCheck = variables.licenseHelper.validateLicense( licenseProperties );
		
		// If nothing changed and their license is already valid, I'm not sure why they clicked "Submit", but let's just ignore it.  Nothing to save!
		// This is mostly to prevent accidental activations from getting used up.
		if( props.email == req.email && 
			props.licenseKey == req.licenseKey && 
			props.serverType == req.serverType && 
			licenseValidCheck.isValid 
		) {
			return 'redirect:overview';			
		}
		
		// Save data from form.
		props.email 		= req.email;
		props.licenseKey 	= req.licenseKey;
		props.serverType	= req.serverType;
		setLicenseProperties( props );
		
		// Only activate non-trial license with a license key and E-mail provided.
		if( req.serverType != 'trial' && len( req.licenseKey ) && len( req.email ) ) {
			// Phone home and store activation details if successful
			activateLicense( req.email, req.licenseKey, req.serverType, arguments.lang );
		}
		
		return 'redirect:overview';
	}

	/*************************************** PRIVATE ******************************************/
	
	private struct function getLicenseProperties() {
		var props = {};
		
		var licensePropertiesPath = getLicensePropertiesPath();
		// Like the light in the fridge-- always on when you're looking
		if( !fileExists( licensePropertiesPath ) ) {
			directoryCreate( getDirectoryFromPath( licensePropertiesPath ), true, true );
			fileWrite( licensePropertiesPath, '' );
		}
		
		var fis 			= createObject( 'java', 'java.io.FileInputStream' ).init( getLicensePropertiesPath() );
		var propertyFile 	= createObject( 'java', 'java.util.Properties' ).init();
		propertyFile.load( fis );
		fis.close();
		
		props.email          = propertyFile.getProperty( 'email', '' );
		props.licenseKey     = propertyFile.getProperty( 'licenseKey', '' );
		props.serverType     = propertyFile.getProperty( 'serverType', '' );
		props.activationCode = propertyFile.getProperty( 'activationCode', '' );
		
		return props;		
	}

	private function setLicenseProperties( required struct props ) {
		
		var fos 			= createObject( 'java', 'java.io.FileOutputStream' ).init( getLicensePropertiesPath() );
		var propertyFile 	= createObject( 'java', 'java.util.Properties' );
				
		propertyFile.setProperty( 'email', props.email ?: '' );
		propertyFile.setProperty( 'licenseKey', props.licenseKey ?: '' );
		propertyFile.setProperty( 'serverType', props.serverType ?: '' );
		propertyFile.setProperty( 'activationCode', props.activationCode ?: '' );
		
		propertyFile.store( fos, 'Ortus MongoDb Extension license/activation details' );
		fos.close();
	}

	private function getLicensePropertiesPath() {
		// web context
		if( request.adminType == 'web' ) {
			return expandPath( '/lucee/ortus/mongodb/license.properties' );
		// server context
		} else {
			return expandPath( '/lucee-server/ortus/mongodb/license.properties' );
		}
	}

	function activateLicense( required string email, required string licenseKey, required string serverType, required struct lang ) {
		
		try {
			
			// validation success, contact server with credentials
			http method="POST"
				 url="#variables.activationServer#"
				 result="local.aResults"
				 timeout="30"
				 throwOnError="true"{
	
				httpparam name="licenseKey" 		value="#arguments.licenseKey#"      type="formfield";
				httpparam name="licenseEmail" 		value="#arguments.email#"           type="formfield";
				httpparam name="ipaddress" 			value="#getlocalhostip()#"          type="formfield";
				httpparam name="hostname" 			value="#variables.hostName#"        type="formfield";
				httpparam name="serverType" 		value="#arguments.serverType#"      type="formfield";
				httpparam name="os" 				value="#server.os.name#"            type="formfield";
				httpparam name="osVersion" 			value="#server.os.version#"         type="formfield";
				httpparam name="engineID" 			value="#getLuceeId().server.id#"    type="formfield";
				httpparam name="engineType" 		value="lucee"                       type="formfield";
				httpparam name="productSKU" 		value="#variables.productSKU#"      type="formfield";
				httpparam name="productVersion" 	value="#variables.productVersion#"  type="formfield";
			}
		} catch( any var e ) {
			log text="#lang.errorActivationServer#: #variables.activationServer# #e.message# #e.detail#" file="Ortus MongoDb Extension" type="fatal";
			session.ortus_extension_error = "#lang.errorActivationServer#. #lang.debugData#: #variables.activationServer# #e.message# #e.detail#";
			return;
		}

		// Verify Results
		if( local.aResults.status_code != "200" || !isJSON( local.aResults.filecontent ) ){
			log text="#lang.errorActivationExt#: #local.aResults.fileContent#" file="Ortus MongoDb Extension" type="fatal";
			session.ortus_extension_error = "#lang.errorNoJSON#. #lang.debugData#: #aResults.fileContent#";
			return;
		}

		// Deserialize Results
		local.aData = deserializeJSON( local.aResults.filecontent );

		// Verify Error
		if( local.aData.error ){
			// TODO: Figure out how to pass proper validation messages to the interface
			session.ortus_extension_error = "#lang.errorActivationUnknown#: <strong>#local.aData.messages#</strong>";
			return;
		}

		// If we got here, then we can go ahead an activate the extension
		var props = getLicenseProperties();
		props.activationCode = local.aData.activationCode;		
		setLicenseProperties( props );
	}
	 
}